@@ -1,7 +1,7 @@
-﻿/*
-Copyright (c) 2003-2013, CKSource - Frederico Knabben. All rights reserved.
-For licensing, see LICENSE.html or http://ckeditor.com/license
-*/
+﻿/**
+ * @license Copyright (c) 2003-2013, CKSource - Frederico Knabben. All rights reserved.
+ * For licensing, see LICENSE.html or http://ckeditor.com/license
+ */
 
 /**
  * @fileOverview Defines the {@link CKEDITOR.editor} class, which is the base
@@ -12,60 +12,67 @@ For licensing, see LICENSE.html or http://ckeditor.com/license
  * Represents a DOM object. This class is not intended to be used directly. It
  * serves as the base class for other classes representing specific DOM
  * objects.
- * @constructor
+ *
+ * @class
+ * @mixins CKEDITOR.event
+ * @constructor Creates a domObject class instance.
  * @param {Object} nativeDomObject A native DOM object.
- * @augments CKEDITOR.event
- * @example
  */
-CKEDITOR.dom.domObject = function( nativeDomObject )
-{
-	if ( nativeDomObject )
-	{
+CKEDITOR.dom.domObject = function( nativeDomObject ) {
+	if ( nativeDomObject ) {
 		/**
 		 * The native DOM object represented by this class instance.
-		 * @type Object
-		 * @example
-		 * var element = new CKEDITOR.dom.element( 'span' );
-		 * alert( element.$.nodeType );  // "1"
+		 *
+		 *		var element = new CKEDITOR.dom.element( 'span' );
+		 *		alert( element.$.nodeType ); // '1'
+		 *
+		 * @readonly
+		 * @property {Object}
 		 */
 		this.$ = nativeDomObject;
 	}
 };
 
-CKEDITOR.dom.domObject.prototype = (function()
-{
+CKEDITOR.dom.domObject.prototype = (function() {
 	// Do not define other local variables here. We want to keep the native
 	// listener closures as clean as possible.
 
-	var getNativeListener = function( domObject, eventName )
-	{
-		return function( domEvent )
-		{
-			// In FF, when reloading the page with the editor focused, it may
-			// throw an error because the CKEDITOR global is not anymore
-			// available. So, we check it here first. (#2923)
-			if ( typeof CKEDITOR != 'undefined' )
-				domObject.fire( eventName, new CKEDITOR.dom.event( domEvent ) );
+	var getNativeListener = function( domObject, eventName ) {
+			return function( domEvent ) {
+				// In FF, when reloading the page with the editor focused, it may
+				// throw an error because the CKEDITOR global is not anymore
+				// available. So, we check it here first. (#2923)
+				if ( typeof CKEDITOR != 'undefined' )
+					domObject.fire( eventName, new CKEDITOR.dom.event( domEvent ) );
+			};
 		};
-	};
 
-	return /** @lends CKEDITOR.dom.domObject.prototype */ {
+	return {
 
-		getPrivate : function()
-		{
+		/**
+		 * Get the private `_` object which is bound to the native
+		 * DOM object using {@link #getCustomData}.
+		 *
+		 *		var elementA = new CKEDITOR.dom.element( nativeElement );
+		 *		elementA.getPrivate().value = 1;
+		 *		...
+		 *		var elementB = new CKEDITOR.dom.element( nativeElement );
+		 *		elementB.getPrivate().value; // 1
+		 *
+		 * @returns {Object} The private object.
+		 */
+		getPrivate: function() {
 			var priv;
 
-			// Get the main private function from the custom data. Create it if not
-			// defined.
+			// Get the main private object from the custom data. Create it if not defined.
 			if ( !( priv = this.getCustomData( '_' ) ) )
 				this.setCustomData( '_', ( priv = {} ) );
 
 			return priv;
 		},
 
-		/** @ignore */
-		on  : function( eventName )
-		{
+		// Docs inherited from event.
+		on: function( eventName ) {
 			// We customize the "on" function here. The basic idea is that we'll have
 			// only one listener for a native event, which will then call all listeners
 			// set to the event.
@@ -73,15 +80,13 @@ CKEDITOR.dom.domObject.prototype = (function()
 			// Get the listeners holder object.
 			var nativeListeners = this.getCustomData( '_cke_nativeListeners' );
 
-			if ( !nativeListeners )
-			{
+			if ( !nativeListeners ) {
 				nativeListeners = {};
 				this.setCustomData( '_cke_nativeListeners', nativeListeners );
 			}
 
 			// Check if we have a listener for that event.
-			if ( !nativeListeners[ eventName ] )
-			{
+			if ( !nativeListeners[ eventName ] ) {
 				var listener = nativeListeners[ eventName ] = getNativeListener( this, eventName );
 
 				if ( this.$.addEventListener )
@@ -94,19 +99,16 @@ CKEDITOR.dom.domObject.prototype = (function()
 			return CKEDITOR.event.prototype.on.apply( this, arguments );
 		},
 
-		/** @ignore */
-		removeListener : function( eventName )
-		{
+		// Docs inherited from event.
+		removeListener: function( eventName ) {
 			// Call the original implementation.
 			CKEDITOR.event.prototype.removeListener.apply( this, arguments );
 
 			// If we don't have listeners for this event, clean the DOM up.
-			if ( !this.hasListeners( eventName ) )
-			{
+			if ( !this.hasListeners( eventName ) ) {
 				var nativeListeners = this.getCustomData( '_cke_nativeListeners' );
 				var listener = nativeListeners && nativeListeners[ eventName ];
-				if ( listener )
-				{
+				if ( listener ) {
 					if ( this.$.removeEventListener )
 						this.$.removeEventListener( eventName, listener, false );
 					else if ( this.$.detachEvent )
@@ -119,14 +121,13 @@ CKEDITOR.dom.domObject.prototype = (function()
 
 		/**
 		 * Removes any listener set on this object.
+		 *
 		 * To avoid memory leaks we must assure that there are no
 		 * references left after the object is no longer needed.
 		 */
-		removeAllListeners : function()
-		{
+		removeAllListeners: function() {
 			var nativeListeners = this.getCustomData( '_cke_nativeListeners' );
-			for ( var eventName in nativeListeners )
-			{
+			for ( var eventName in nativeListeners ) {
 				var listener = nativeListeners[ eventName ];
 				if ( this.$.detachEvent )
 					this.$.detachEvent( 'on' + eventName, listener );
@@ -139,49 +140,50 @@ CKEDITOR.dom.domObject.prototype = (function()
 	};
 })();
 
-(function( domObjectProto )
-{
+(function( domObjectProto ) {
 	var customData = {};
 
-	CKEDITOR.on( 'reset', function()
-		{
-			customData = {};
-		});
+	CKEDITOR.on( 'reset', function() {
+		customData = {};
+	});
 
 	/**
 	 * Determines whether the specified object is equal to the current object.
-	 * @name CKEDITOR.dom.domObject.prototype.equals
-	 * @function
+	 *
+	 *		var doc = new CKEDITOR.dom.document( document );
+	 *		alert( doc.equals( CKEDITOR.document ) );	// true
+	 *		alert( doc == CKEDITOR.document );			// false
+	 *
 	 * @param {Object} object The object to compare with the current object.
-	 * @returns {Boolean} "true" if the object is equal.
-	 * @example
-	 * var doc = new CKEDITOR.dom.document( document );
-	 * alert( doc.equals( CKEDITOR.document ) );  // "true"
-	 * alert( doc == CKEDITOR.document );         // "false"
+	 * @returns {Boolean} `true` if the object is equal.
 	 */
-	domObjectProto.equals = function( object )
-	{
-		return ( object && object.$ === this.$ );
+	domObjectProto.equals = function( object ) {
+		// Try/Catch to avoid IE permission error when object is from different document.
+		try {
+			return ( object && object.$ === this.$ );
+		} catch ( er ) {
+			return false;
+		}
 	};
 
 	/**
 	 * Sets a data slot value for this object. These values are shared by all
 	 * instances pointing to that same DOM object.
-	 * <strong>Note:</strong> The created data slot is only guarantied to be available on this unique dom node,
-	 * thus any wish to continue access it from other element clones (either created by clone node or from innerHtml)
-	 * will fail, for such usage, please use {@link CKEDITOR.dom.element::setAttribute} instead.
-	 * @name CKEDITOR.dom.domObject.prototype.setCustomData
-	 * @function
+	 *
+	 * **Note:** The created data slot is only guarantied to be available on this unique dom node,
+	 * thus any wish to continue access it from other element clones (either created by
+	 * clone node or from `innerHtml`) will fail, for such usage, please use
+	 * {@link CKEDITOR.dom.element#setAttribute} instead.
+	 *
+	 *		var element = new CKEDITOR.dom.element( 'span' );
+	 *		element.setCustomData( 'hasCustomData', true );
+	 *
 	 * @param {String} key A key used to identify the data slot.
 	 * @param {Object} value The value to set to the data slot.
 	 * @returns {CKEDITOR.dom.domObject} This DOM object instance.
-	 * @see CKEDITOR.dom.domObject.prototype.getCustomData
-	 * @example
-	 * var element = new CKEDITOR.dom.element( 'span' );
-	 * element.setCustomData( 'hasCustomData', true );
+	 * @chainable
 	 */
-	domObjectProto.setCustomData = function( key, value )
-	{
+	domObjectProto.setCustomData = function( key, value ) {
 		var expandoNumber = this.getUniqueId(),
 			dataSlot = customData[ expandoNumber ] || ( customData[ expandoNumber ] = {} );
 
@@ -192,47 +194,47 @@ CKEDITOR.dom.domObject.prototype = (function()
 
 	/**
 	 * Gets the value set to a data slot in this object.
-	 * @name CKEDITOR.dom.domObject.prototype.getCustomData
-	 * @function
+	 *
+	 *		var element = new CKEDITOR.dom.element( 'span' );
+	 *		alert( element.getCustomData( 'hasCustomData' ) );		// e.g. 'true'
+	 *		alert( element.getCustomData( 'nonExistingKey' ) );		// null
+	 *
 	 * @param {String} key The key used to identify the data slot.
 	 * @returns {Object} This value set to the data slot.
-	 * @see CKEDITOR.dom.domObject.prototype.setCustomData
-	 * @example
-	 * var element = new CKEDITOR.dom.element( 'span' );
-	 * alert( element.getCustomData( 'hasCustomData' ) );  // e.g. 'true'
 	 */
-	domObjectProto.getCustomData = function( key )
-	{
+	domObjectProto.getCustomData = function( key ) {
 		var expandoNumber = this.$[ 'data-cke-expando' ],
 			dataSlot = expandoNumber && customData[ expandoNumber ];
 
-		return dataSlot && dataSlot[ key ];
+		return ( dataSlot && key in dataSlot ) ? dataSlot[ key ] : null;
 	};
 
 	/**
-	 * @name CKEDITOR.dom.domObject.prototype.removeCustomData
+	 * Removes the value in data slot under given `key`.
+	 *
+	 * @param {String} key
+	 * @returns {Object} Removed value or `null` if not found.
 	 */
-	domObjectProto.removeCustomData = function( key )
-	{
+	domObjectProto.removeCustomData = function( key ) {
 		var expandoNumber = this.$[ 'data-cke-expando' ],
 			dataSlot = expandoNumber && customData[ expandoNumber ],
-			retval = dataSlot && dataSlot[ key ];
+			retval, hadKey;
 
-		if ( typeof retval != 'undefined' )
+		if ( dataSlot ) {
+			retval = dataSlot[ key ];
+			hadKey = key in dataSlot;
 			delete dataSlot[ key ];
+		}
 
-		return retval || null;
+		return hadKey ? retval : null;
 	};
 
 	/**
 	 * Removes any data stored on this object.
 	 * To avoid memory leaks we must assure that there are no
 	 * references left after the object is no longer needed.
-	 * @name CKEDITOR.dom.domObject.prototype.clearCustomData
-	 * @function
 	 */
-	domObjectProto.clearCustomData = function()
-	{
+	domObjectProto.clearCustomData = function() {
 		// Clear all event listeners
 		this.removeAllListeners();
 
@@ -243,12 +245,10 @@ CKEDITOR.dom.domObject.prototype = (function()
 	/**
 	 * Gets an ID that can be used to identiquely identify this DOM object in
 	 * the running session.
-	 * @name CKEDITOR.dom.domObject.prototype.getUniqueId
-	 * @function
+	 *
 	 * @returns {Number} A unique ID.
 	 */
-	domObjectProto.getUniqueId = function()
-	{
+	domObjectProto.getUniqueId = function() {
 		return this.$[ 'data-cke-expando' ] || ( this.$[ 'data-cke-expando' ] = CKEDITOR.tools.getNextNumber() );
 	};
 
